#
# PST module Empty - clear some pages
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
# Options:
#    this defines pagemap which pages should be blanked
# e|even        blank even pages
# o|odd         blank odd pages
# p|pages = (pagespec1,pagespec2,...)
#        where pagespec is one of
#            <int>   blank page of this number
#            _<int>  as ^ but counted from end
#            <int> - <int>    range, if left side omitted, then from 1, if right then to end


package Empty;
use OM;
@ISA = qw(OM);
use strict;



sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("Empty");
	$self->{PAGE_SPEC} = {};
	$self->{EVEN} = 0;
	$self->{ODD} = 0;
	$self->{EMPTY} = {};

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;
	my $str = "";
	my $j = 1;

	$self->log(4,"parsing options");
	while (my $s = shift) { $str .= $s }
	$str =~ s/\s//g;
	foreach my $e (split /;/, $str) {
		if ($e =~ /^(p|pages)\=\(([^\)]*)\)$/) {
			foreach my $p (split /,/, $2) {
				if ($p =~ /^_?\d+|(_?\d+)?-(_?\d+)?$/) {
					$self->{PAGE_SPEC}{$j++} = $p;
				} else {
					$self->die("invalid pagespec ($p)\n");
				}
			}
		} elsif ($e =~ /^(e|even)$/) {
			$self->{EVEN} = 1;
		} elsif ($e =~ /^(o|odd)$/) {
			$self->{ODD} = 1;
		} else {
			$self->die("bad option ($str)\n");
		}
	}
}

sub run {
	my $self = shift;

	$self->previous->run;
	for my $i (1..$self->{NUM_PAGES}) {
		$self->{EMPTY}{$i} = 0;
		# just munch page_spec and find if a page will be blanked
		foreach my $ps (values %{$self->{PAGE_SPEC}}){
			$self->{EMPTY}{$i} = 1 if
				$ps =~ /^\d+$/ && $i == $ps
				or $ps =~ /^_(\d+)$/ && $i == $self->{NUM_PAGES}-$1+1
				or $ps =~ /^(\d*)-(\d*)$/ && $i >= (($1 eq "")? 1 : $1)
					&& $i <= (($2 eq "")? $self->{NUM_PAGES} : $2);
		}
	}

	# mark all even pages
	if ($self->{EVEN}) {
		for (my $i=2; $i<=$self->{NUM_PAGES}; $i+=2) {
			$self->{EMPTY}{$i} = 1;
		}
	}
	# mark all odd pages
	if ($self->{ODD}) {
		for (my $i=1; $i<=$self->{NUM_PAGES}; $i+=2) {
			$self->{EMPTY}{$i} = 1;
		}
	}
}

sub write_page {
	my $self = shift;
	my $page = shift;

	if ($self->{EMPTY}{$page}) {   # blank page - only show empty page
		$self->log(4,"writing page $page blank");
		my $f = $self->{OUTPUT};
		print $f "showpage\n";
	} else { $self->previous->write_page($page) }
}



1; # no init...
