#
# PST module Flip - flip page horizontally or vertically
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
# Options:
# fm|flip_mode = <mode>
#    where mode is one of h|horizontal  or  v|vertical


package Flip;
use OM;
@ISA = qw(OM);
use strict;




sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;
	$self->name("Flip");
	$self->{FLIP_MODE} = undef;  # 0=horizontal, 1=vertical
	
	return bless($self,$class);
}

sub parse_options {
	my $self = shift;
	my $str;

	$self->log(4,"parsing options");
	while ($str = shift) {
		if ($str !~ /^\s*(fm|flip_mode)\s*\=\s*(h|v|horizontal|vertical)\s*$/) {
			$self->die("bad options\n");
		}
		if ($2 eq "h" || $2 eq "horizontal") { $self->{FLIP_MODE} = 0 }
		else { $self->{FLIP_MODE} = 1 }
	}
	if (not defined $self->{FLIP_MODE}) {
		$self->die("I need some options\n");
	}
	$self->log(2,($self->{FLIP_MODE} ? "vertical" : "horizontal")," flip");
}

sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $h = $self->{OUTPUT};
	my $pw = $self->{PAGE_WIDTH}{$page};
	my $ph = $self->{PAGE_HEIGHT}{$page};

	$self->log(4,"writing page $page prolog (with some scaling)");
	print $h "userdict/PSTdict save put\nPSTmatrix setmatrix\n";   # transform matrix manipulation - save state
	if ($self->{FLIP_MODE} == 0) {  # horizontal
		print $h
			# we must move the page since it will be left from visible area
			$self->{PAGE_WIDTH}{$page}," 0 translate\n",  
			"-1 1 scale\n";  # do the horizontal flip 
	} else {  # vertical
		print $h
			# now the page is below visible area ... 
			"0 ",$self->{PAGE_HEIGHT}{$page}," translate\n",
			"1 -1 scale\n"; # ... and flip it
	}
	print $h
		"userdict/PSTmatrix matrix currentmatrix put\n",
		#"userdict/PSTclip{0 0 moveto\n",
		#$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
		#"closepath}put initclip\n",
		"PSTxform concat\n";
}


sub write_page_epilog {
	my $self = shift;
	my $page = shift;
	my $h = $self->{OUTPUT};

	$self->log(4,"writing page $page epilog");
	print $h "PSTdict restore\n";   # back to normal...
}


1;  # no init
