#
# PST module Select - select some pages from input
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
# options:
# e|even       write even pages
# o|odd        dtto
# r|reverse    print pages in reverse order
# p|pages = (ps1,ps2,...)
#     where ps is one of 
#            <int>       print page of this number
#            _<int>      dtto but counted from end
#            <int> - <int>    range, if one side omitted, 1 or page_num will be supplied
#                             ( the same '_' rule apply)


package Select;
use OM;
@ISA = qw(OM);
use strict;
use integer;


sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;
	$self->name("Select");
	$self->{ODD} = 0;
	$self->{EVEN} = 0;
	$self->{REVERSE} = 0;
	$self->{PAGE_SPEC} = {};
	$self->{ORIG_PAGE} = {};
	$self->{P} = {};
	$self->{Q} = {};

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;
	my $p=$self->{P};
	my $q=$self->{Q};

	$self->log(4,"parsing options");
	while (my $str = shift) {
		# god, this is ugly... I must rewrite it
		foreach my $e (split /;/, $str) {
			if ($e =~ /^\s*(e|even)\s*$/) {
				$self->{EVEN} = 1;
			} elsif ($e =~ /^\s*(o|odd)\s*$/) {
				$self->{ODD} = 1;
			} elsif ($e =~ /^\s*(r|reverse)\s*$/) {
				$self->{REVERSE} = 1;
			} elsif ($e =~ /^\s*(p|pages)\s*\=\s*\(([^\)]+)\)\s*$/) {
				my $i = 1;
				for my $pe (split /,/, $2) {
					if ($pe =~ /^\s*(\d+)\s*$/) {
						$p->{$i} = $1;
					} elsif ($pe =~ /^\s*_(\d+)\s*$/) {
						$p->{$i} = -$1;
					} elsif ($pe =~ /^\s*(.*)-(.*)\s*$/) {
						my $a = $1;
						my $b = $2;
						if ($a =~ /^\d+\s*$/) {
							$p->{$i} = $a;
						} elsif ($a =~ /^_\d+\s*$/) {
							$p->{$i} = -$a;
						} elsif ($a =~ /^\s*$/) {
							$p->{$i} = 1;
						} elsif ($a =~ /^_\s*$/) {
							$p->{$i} = -1;
						} else {
							$self->die("bad pagespec ($a)\n");
						}
						if ($b =~ /^\d+\s*$/) {
							$q->{$i} = $b;
						} elsif ($b =~ /^_\d+\s*$/) {
							$q->{$i} = -$b;
						} elsif ($b =~ /^\s*$/) {
							$q->{$i} = -1;
						} elsif ($b =~ /^_\s*$/) {
							$q->{$i} = 1;
						} else {
							$self->die("bad pagespec ($b)\n");
						}
					} else {
						$self->die("bad pagespec\n");
					}
					++ $i;
				}
			} else {
				$self->die("bad options!\n");
			}
		}
	}
	if (!$self->{ODD} && !$self->{EVEN} && !$self->{P}) {
		$self->die("I need some options\n");
	}
}

sub calculate {
        my $self = shift;
	my $p=$self->{P};
	my $q=$self->{Q};

	my $pr = $self->previous;
	$pr->calculate;
	for(my $i=1; $i<=$pr->{NUM_PAGES}; $i++) {
		$self->{PREV_PAGE_WIDTH}{$i} = $pr->{PAGE_WIDTH}{$i};
		$self->{PREV_PAGE_HEIGHT}{$i} = $pr->{PAGE_HEIGHT}{$i};
	}
	$self->{PREV_PAGES} = $pr->{NUM_PAGES};

	my $j=1;
	my $pgs = $self->{PREV_PAGES};
	for my $i (keys %{$p}) {
		my $a = $p->{$i};
		my $b = $q->{$i};
		$a = $pgs + $a +1 if $a<0; # from end

		if (not defined $b) {  # single page
			$self->{PAGE_SPEC}{$j++} = $a;
		} else { # it is range
			$b = $pgs + $b +1 if $b<0;
			if ($a>$b) {
				for (my $k=$a; $k>=$b; $k--) {
					$self->{PAGE_SPEC}{$j++} = $k;
				}
			} else {
				for (my $k=$a; $k<=$b; $k++) {
					$self->{PAGE_SPEC}{$j++} = $k;
				}
			}
		}
	}

	my $i = 1;
	while ($self->page_size($i)) {   # expect true if page $i will be produced, otherwise end
		++ $i;
	}
	$self->{NUM_PAGES} = $i - 1;
}

sub page_size {
	my $self = shift;
	my $page = shift;

	if ($self->{ODD} || $self->{EVEN}) {
		my $pp;
		if ($self->{ODD}) {
			$pp = $page*2 - 1;
		} else { # EVEN
			$pp = $page*2;
		}
		return 0 unless $pp>0 & $pp<=$self->{PREV_PAGES};
		$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$pp};
		$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$pp};
		$self->{ORIG_PAGE}{$page} = $pp;
		return 1;
	} else {  # we have PAGE_SPEC
		my $pp = $self->{PAGE_SPEC}{$page};
		return 0 if not defined $pp;
		$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$pp};
		$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$pp};
		$self->{ORIG_PAGE}{$page} = $pp;
		return 1;
	}
	return 0;
}

sub write_page {
	my $self = shift;
	my $page = shift;

	$page = $self->{NUM_PAGES} - $page +1 if $self->{REVERSE};
	$self->previous->write_page($self->{ORIG_PAGE}{$page});
	# print $page," ",$self->{ORIG_PAGE}{$page},"\n";
}


1; # no init...
