#
# PST lib sizes - translates length units into PS points, return paper sizes
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
#
# unit_size(string)   where string is in format "number unit"
#      returns value translated into points, or "ERROR errorcause" on error
#
# paper_size(papername)  where paper is name of common paper format (case insensitive)
#      returns 2 field array, [0] is width, [1] is height
#       or "ERROR cause" on error

package sizes;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(unit_size paper_size);  # export these symbols
use strict;

# how many points are these units
my %unit_sizes = (
	pt	=> 1,		# 1 pt = 1/72 in
	pc	=> 12,
	in	=> 72,
	cm	=> 72 / 2.54,	# 1 in = 2.54 cm
	mm	=> 72 / 25.4,
	dm	=> 72 / 0.254,
);

sub unit_size {
	my $s = shift;
	$s =~ /^\s*(-?\d+(\.\d+)?)\s*([a-z]+)\s*$/ or return "ERROR bad format";
	defined $unit_sizes{$3} or return "ERROR unknow unit";
	return $1 * $unit_sizes{$3};
}


my %paper_sizes = (
	a3		=> [842,1191],
	a4		=> [595,842],
	a5		=> [421,595],
	b5		=> [516,729],
	letter		=> [612,792],
	legal		=> [612,1008],
	ledger		=> [1224,792],
	tabloid		=> [792,1224],
	statement	=> [396,612],
	executive	=> [540,720],
	folio		=> [612,936],
	quarto		=> [610,780],
	'10x14'		=> [720,1008],
);

sub paper_size {
	my $s = shift;
	$s =~ tr/[A-Z]/[a-z]/;
	defined $paper_sizes{$s} or return "ERROR unknow paper";
	return @{$paper_sizes{$s}};   # return array
}


1;   # no init

